package mschmidt.komo.datatools;
/*
 * Wizard page that allows user to enter options for the database backup
 * process.
 */

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import mschmidt.komo.KomoPlugin;
import mschmidt.komo.ApplicationImages;

public class BackupWizardPage extends WizardPage {

	/*
	 * Instantiate Text variables because their listener require it.
	 */
	private Text backupEngineText;

	private Text fileNameText;

	/**
	 * Constructor to create the wizard and initialize variables.
	 */
	public BackupWizardPage() {
		super("backupPage");
		setTitle("Database Backup Wizard");
		setDescription("Database Backup Options");
		setImageDescriptor(KomoPlugin
				.getImageDescriptor(ApplicationImages.DB_BACKUP32_ICON));
		setPageComplete(false);
	}

	/**
	 * Create contents of the wizard.  Values entered on this page are 
	 * recorded in the data model.
	 * 
	 * @param parent -
	 *            the composite's parent
	 */
	public final void createControl(final Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.verticalSpacing = 3;
		gridLayout.numColumns = 3;
		gridLayout.horizontalSpacing = 5;
		container.setLayout(gridLayout);
		//
		setControl(container);

		final Label backupFileLabel = new Label(container, SWT.NONE);
		backupFileLabel.setToolTipText("Create/select backup file to use");
		backupFileLabel.setText("Backup File");

		fileNameText = new Text(container, SWT.BORDER);
		fileNameText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				((BackupWizard) getWizard()).data.backupFile = fileNameText
						.getText();
				CheckPage();
			}
		});
		fileNameText.setLayoutData(new GridData(GridData.FILL, GridData.CENTER,
				true, false));
		fileNameText.setText(((BackupWizard) getWizard()).data.backupFile);

		Button fileBrowseButton = new Button(container, SWT.NONE);
		fileBrowseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				FileDialog fileBrowseDialog = new FileDialog(getShell(),
						SWT.SAVE);
				fileBrowseDialog.setFilterNames(new String[] {
						"PostgreSQL Custom Archive files (*.pgb)",
						"Bak Archive files (*.bak)",
						"Backup Archive files (*.backup)",
						"Tar Archive files (*.tar)", "All files (*.*)" });
				fileBrowseDialog.setFilterExtensions(new String[] { "*.pgb",
						"*.bak", "*.backup", "*.tar", "*.*" });
				String stReturn = fileBrowseDialog.open();
				if (stReturn != null && stReturn.length() > 0) {
					fileNameText.setText(stReturn);
				}
			}
		});
		fileBrowseButton.setText("Browse...");

		final Label backupEngineLabel = new Label(container, SWT.NONE);
		backupEngineLabel
				.setToolTipText("The PostgreSQL backup engine (pg_dump file)");
		backupEngineLabel.setText("Backup Engine");

		backupEngineText = new Text(container, SWT.BORDER);
		backupEngineText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				((BackupWizard) getWizard()).data.backupEngine = backupEngineText
						.getText();
				CheckPage();
			}
		});
		backupEngineText.setLayoutData(new GridData(GridData.FILL,
				GridData.CENTER, true, false));
		backupEngineText
				.setText(((BackupWizard) getWizard()).data.backupEngine);

		final Button engineBrowseButton = new Button(container, SWT.NONE);
		engineBrowseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				FileDialog engineBrowseDialog = new FileDialog(getShell(),
						SWT.SAVE);
				engineBrowseDialog.setFilterNames(new String[] {
						"PostgreSQL backup engine (pg_dump.exe)",
						".exe files (*.exe)", "All files (*.*)" });
				engineBrowseDialog.setFilterExtensions(new String[] {
						"pg_dump.exe", "*.exe", "*.*" });
				String stReturn = engineBrowseDialog.open();
				if (stReturn != null && stReturn.length() > 0) {
					backupEngineText.setText(stReturn);
					// Warn the user if pg_dump.exe was not selected.
					if (!stReturn.endsWith("pg_dump.exe")) {
						MessageDialog.openWarning(getShell(), 
								"Backup Engine Warning",
								"The selected file does not appear to be\nthe PostgreSQL backup engine (pg_dump.exe)");
					}
				}
			}
		});
		engineBrowseButton.setText("Browse...");

		final Label formatLabel = new Label(container, SWT.NONE);
		formatLabel
				.setToolTipText("Select the format to be used for the backup file (Custom recommended)");
		formatLabel.setText("File Format");

		final Group formatGroup = new Group(container, SWT.NONE);
		final GridLayout formatGridLayout = new GridLayout();
		formatGridLayout.marginHeight = 3;
		formatGridLayout.horizontalSpacing = 10;
		formatGridLayout.numColumns = 2;
		formatGroup.setLayout(formatGridLayout);

		final Button customButton = new Button(formatGroup, SWT.RADIO);
		customButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				if (customButton.getSelection()) {
					((BackupWizard) getWizard()).data.backupFormat = "Custom";
				} else {
					((BackupWizard) getWizard()).data.backupFormat = "Tar";
				}
			}
		});
		customButton.setText("Custom");

		final Button tarButton = new Button(formatGroup, SWT.RADIO);
		tarButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				if (tarButton.getSelection()) {
					((BackupWizard) getWizard()).data.backupFormat = "Tar";
				} else {
					((BackupWizard) getWizard()).data.backupFormat = "Custom";
				}
			}
		});
		tarButton.setText(".tar");

		if (((BackupWizard) getWizard()).data.backupFormat.equals("Tar")) {
			tarButton.setSelection(true);
		} else {
			customButton.setSelection(true);
		}
		new Label(container, SWT.NONE);

		final Button withOidsButton = new Button(container, SWT.CHECK);
		withOidsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((BackupWizard) getWizard()).data.doWithOids = withOidsButton
						.getSelection();
			}
		});
		withOidsButton
				.setToolTipText("Check to include Object Identifiers (OIDs)");
		withOidsButton.setText("With OIDs");
		withOidsButton
				.setSelection(((BackupWizard) getWizard()).data.doWithOids);

		final Button disableDollarButton = new Button(container, SWT.CHECK);
		disableDollarButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((BackupWizard) getWizard()).data.disableDollar = disableDollarButton
						.getSelection();
			}
		});
		disableDollarButton
				.setToolTipText("Check to disable dollar-quote format in database functions");
		disableDollarButton.setText("Disable Dollar Quoting");
		disableDollarButton
				.setSelection(((BackupWizard) getWizard()).data.disableDollar);
		new Label(container, SWT.NONE);

		final Button verboseButton = new Button(container, SWT.CHECK);
		verboseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((BackupWizard) getWizard()).data.doVerbose = verboseButton
						.getSelection();
			}
		});
		verboseButton
				.setToolTipText("Check to display detailed output about Backup operation");
		verboseButton.setText("Verbose");
		verboseButton.setSelection(((BackupWizard) getWizard()).data.doVerbose);

		final Button ignoreVersionButton = new Button(container, SWT.CHECK);
		ignoreVersionButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((BackupWizard) getWizard()).data.ignoreVersion = ignoreVersionButton
						.getSelection();
			}
		});
		ignoreVersionButton
				.setToolTipText("Check to ignore database version warnings");
		ignoreVersionButton.setText("Ignore Version");
		ignoreVersionButton
				.setSelection(((BackupWizard) getWizard()).data.ignoreVersion);
		new Label(container, SWT.NONE);

	}
	
	/*
	 * Method to check if page is complete.
	 */
	private void CheckPage() {
		if (((BackupWizard) getWizard()).data.backupFile != null
				&& ((BackupWizard) getWizard()).data.backupFile
						.length() > 0
				&& ((BackupWizard) getWizard()).data.backupEngine != null
				&& ((BackupWizard) getWizard()).data.backupEngine
						.length() > 0) {
			setPageComplete(true);
		} else {
			setPageComplete(false);
		}
		
	}

}
